@@ -29,6 +29,8 @@ LOGGER = logging.getLogger(__package__).getChild("Mapper")
 class Mapper(Generic[ValueType, CandidateType, ContextType]):
     """Optimal value-candidate matching.
 
+    For an introduction to mapping, see the :ref:`mapping-primer` page.
+
     Args:
         score_function: A callable which accepts a value `k` and an ordered collection of candidates `c`, returning a
             score ``s_i`` for each candidate `c_i` in `c`. Default: ``s_i = float(k == c_i)``. Higher=better match.
@@ -44,18 +46,19 @@ class Mapper(Generic[ValueType, CandidateType, ContextType]):
         cardinality: Desired cardinality for mapped values. Derive for each matching if ``None``.
         enable_verbose_logging: If ``True``, enable verbose logging for the :meth:`apply` function. Has no effect when
             the log level is above ``logging.DEBUG``.
-
-    See Also:
-        The :ref:`mapping-primer` page.
     """
 
     def __init__(
         self,
-        score_function: Union[str, ScoreFunction] = "equality",
+        score_function: Union[str, ScoreFunction[ValueType, CandidateType, ContextType]] = "equality",
         score_function_kwargs: Dict[str, Any] = None,
-        filter_functions: Iterable[Tuple[Union[str, FilterFunction], Dict[str, Any]]] = (),
+        filter_functions: Iterable[
+            Tuple[Union[str, FilterFunction[ValueType, CandidateType, ContextType]], Dict[str, Any]]
+        ] = (),
         min_score: float = 1.00,
-        overrides: Union[InheritedKeysDict, Dict[ValueType, CandidateType]] = None,
+        overrides: Union[
+            InheritedKeysDict[ContextType, ValueType, CandidateType], Dict[ValueType, CandidateType]
+        ] = None,
         unmapped_values_action: ActionLevel.ParseType = ActionLevel.IGNORE,
         unknown_user_override_action: ActionLevel.ParseType = ActionLevel.RAISE,
         cardinality: Optional[Cardinality.ParseType] = Cardinality.ManyToOne,
@@ -64,14 +67,14 @@ class Mapper(Generic[ValueType, CandidateType, ContextType]):
         self._score = get_by_full_name(score_function, sf) if isinstance(score_function, str) else score_function
         self._score_kwargs = score_function_kwargs or {}
         self._min_score = min_score
-        self._overrides: Union[InheritedKeysDict, Dict[ValueType, CandidateType]] = (
-            overrides if isinstance(overrides, InheritedKeysDict) else (overrides or {})
-        )
+        self._overrides: Union[
+            InheritedKeysDict[ContextType, ValueType, CandidateType], Dict[ValueType, CandidateType]
+        ] = (overrides if isinstance(overrides, InheritedKeysDict) else (overrides or {}))
         self._context_sensitive_overrides = isinstance(self._overrides, InheritedKeysDict)
         self._unmapped_action: ActionLevel = ActionLevel.verify(unmapped_values_action)
         self._bad_candidate_action: ActionLevel = ActionLevel.verify(unknown_user_override_action)
         self._cardinality = None if cardinality is None else Cardinality.parse(cardinality, strict=True)
-        self._filters: List[Tuple[FilterFunction, Dict[str, Any]]] = [
+        self._filters: List[Tuple[FilterFunction[ValueType, CandidateType, ContextType], Dict[str, Any]]] = [
             ((get_by_full_name(func, mf) if isinstance(func, str) else func), kwargs)
             for func, kwargs in filter_functions
         ]
@@ -82,7 +85,7 @@ class Mapper(Generic[ValueType, CandidateType, ContextType]):
         values: Iterable[ValueType],
         candidates: Iterable[CandidateType],
         context: ContextType = None,
-        override_function: UserOverrideFunction = None,
+        override_function: UserOverrideFunction[ValueType, CandidateType, ContextType] = None,
         **kwargs: Any,
     ) -> DirectionalMapping[ValueType, CandidateType]:
         """Map values to candidates.
@@ -153,7 +156,7 @@ class Mapper(Generic[ValueType, CandidateType, ContextType]):
         values: Iterable[ValueType],
         candidates: Iterable[CandidateType],
         context: ContextType = None,
-        override_function: UserOverrideFunction = None,
+        override_function: UserOverrideFunction[ValueType, CandidateType, ContextType] = None,
         **kwargs: Any,
     ) -> pd.DataFrame:
         """Compute likeness scores.
@@ -186,21 +189,30 @@ class Mapper(Generic[ValueType, CandidateType, ContextType]):
             dtype=float,
         )
 
+        extra = f" in {context=}" if context else ""
+        if LOGGER.isEnabledFor(logging.DEBUG):
+            candidates = tuple(scores.columns)
+            values = tuple(scores.index)
+            LOGGER.debug(f"Begin computing match scores for {values=}{extra} to {candidates=} using {self._score}.")
+
         unmapped_values = self._handle_overrides(scores, context, override_function)
 
-        extra = f" in {context=}" if context else ""
         for value in unmapped_values:
-            if LOGGER.isEnabledFor(logging.DEBUG):
-                candidates = set(scores.columns)
-                LOGGER.debug(f"Begin computing match scores for {value=}{extra} to {candidates=} using {self._score}.")
-
+            if self.verbose and LOGGER.isEnabledFor(logging.DEBUG):
+                LOGGER.debug(f"Apply filters for {value=}.")
             filtered_candidates = self._apply_filters(value, scores.columns, context, kwargs)
+
+            if self.verbose and LOGGER.isEnabledFor(logging.DEBUG):
+                LOGGER.debug(f"Compute match scores for {value=}.")
             scores_for_value = self._score(value, filtered_candidates, context, **self._score_kwargs, **kwargs)
             for score, candidate in zip(scores_for_value, filtered_candidates):
                 scores.loc[value, candidate] = score
 
         if LOGGER.isEnabledFor(logging.DEBUG):
-            LOGGER.debug(f"Computed {scores.size} match scores in {format_perf_counter(start)}:\n{scores.to_string()}")
+            LOGGER.debug(
+                f"Computed {len(scores.index)}x{len(scores.columns)} "
+                f"match scores in {format_perf_counter(start)}:\n{scores.to_string()}"
+            )
         return scores
 
     def to_directional_mapping(
@@ -248,8 +260,8 @@ class Mapper(Generic[ValueType, CandidateType, ContextType]):
 
     @property
     def context_sensitive_overrides(self) -> bool:
-        """Return ``True`` if overrides are context sensitive."""
-        return self._context_sensitive_overrides
+        """Return ``True`` if there are no overrides, or if the overrides are context sensitive."""
+        return not self._overrides or self._context_sensitive_overrides
 
     @property
     def verbose(self) -> bool:
@@ -260,30 +272,28 @@ class Mapper(Generic[ValueType, CandidateType, ContextType]):
         self,
         scores: pd.DataFrame,
         context: Optional[ContextType],
-        override_function: Optional[UserOverrideFunction],
-    ) -> Set[ValueType]:
+        override_function: Optional[UserOverrideFunction[ValueType, CandidateType, ContextType]],
+    ) -> List[ValueType]:
         def apply(v: ValueType, oc: CandidateType) -> None:
-            if self.cardinality and self.cardinality.one_left:
-                scores.loc[:, oc] = -np.inf  # Prevent candidate reuse
-            if self.cardinality and self.cardinality.one_right:
-                scores.loc[v, :] = -np.inf  # Prevent value reuse
-
+            scores.loc[v, :] = -np.inf
             scores.loc[v, oc] = np.inf
-            unmapped_values.discard(v)
+            unmapped_values.remove(v)
 
-        unmapped_values = set(scores.index)
-        for value, override_candidate in self._get_static_overrides(scores.index, context):
-            apply(value, override_candidate)
+        unmapped_values = list(scores.index)
 
         if override_function:
             for value, override_candidate in self._get_function_overrides(
                 override_function, scores.index, scores.columns, context
             ):
-                LOGGER.debug(
-                    f"Using override {repr(value)} -> {repr(override_candidate)} returned by {override_function}."
-                )
+                if LOGGER.isEnabledFor(logging.DEBUG):
+                    LOGGER.debug(
+                        f"Using override {repr(value)} -> {repr(override_candidate)} returned by {override_function}."
+                    )
                 apply(value, override_candidate)
 
+        for value, override_candidate in self._get_static_overrides(unmapped_values, context):
+            apply(value, override_candidate)
+
         return unmapped_values
 
     def _get_static_overrides(
@@ -291,8 +301,10 @@ class Mapper(Generic[ValueType, CandidateType, ContextType]):
         values: Iterable[ValueType],
         context: Optional[ContextType],
     ) -> List[Tuple[ValueType, CandidateType]]:
-        overrides: Dict[ValueType, CandidateType]  # Type on override check done during init
+        if not self._overrides:
+            return []  # pragma: no cover
 
+        overrides: Dict[ValueType, CandidateType]  # Type on override check done during init
         if context is None:
             if self._context_sensitive_overrides:  # pragma: no cover
                 raise TypeError("Must pass a context when using context-sensitive overrides.")
@@ -306,7 +318,7 @@ class Mapper(Generic[ValueType, CandidateType, ContextType]):
 
     def _get_function_overrides(
         self,
-        func: UserOverrideFunction,
+        func: UserOverrideFunction[ValueType, CandidateType, ContextType],
         values: Iterable[ValueType],
         candidates: Iterable[CandidateType],
         context: Optional[ContextType],
@@ -320,8 +332,8 @@ class Mapper(Generic[ValueType, CandidateType, ContextType]):
                 continue
             if self.unknown_user_override_action is not ActionLevel.IGNORE and user_override not in candidates:
                 msg = (
-                    f"The user-defined override function {func} returned an unknown candidate {repr(user_override)} for"
-                    f" {value=}. If this is intended behaviour, set unknown_user_override_action = 'ignore' to allow."
+                    f"The user-defined override function {func} returned an unknown candidate={repr(user_override)} for"
+                    f" {value=}. If this is intended behaviour, set unknown_user_override_action='ignore' to allow."
                 )
                 if self.unknown_user_override_action is ActionLevel.RAISE:
                     LOGGER.error(msg)
@@ -363,7 +375,7 @@ class Mapper(Generic[ValueType, CandidateType, ContextType]):
         score = self._score
         return f"{tname(self)}({score=} >= {self._min_score}, {len(self._filters)} filters)"
 
-    def copy(self) -> "Mapper":
+    def copy(self) -> "Mapper[ValueType, CandidateType, ContextType]":
         """Make a copy of this ``Mapper``."""
         return Mapper(
             score_function=self._score,
