@@ -1,20 +1,22 @@
 import logging
 import warnings
+from datetime import timedelta
 from pathlib import Path
 from time import perf_counter
 from typing import Any, Dict, Generic, Iterable, List, Optional, Set, Tuple, Type, Union
 
 import numpy
+import pandas as pd
 
 from rics._internal_support.types import PathLikeType
 from rics.mapping import DirectionalMapping, Mapper
-from rics.mapping.exceptions import MappingError, MappingWarning, UserMappingError
+from rics.mapping.exceptions import MappingError, MappingWarning
+from rics.mapping.types import UserOverrideFunction
 from rics.performance import format_perf_counter
-from rics.translation import factory
+from rics.translation import _config_utils, factory
 from rics.translation.dio import DataStructureIO, resolve_io
 from rics.translation.exceptions import ConnectionStatusError, TooManyFailedTranslationsError
 from rics.translation.fetching import Fetcher
-from rics.translation.fetching.exceptions import UnknownSourceError
 from rics.translation.fetching.types import IdsToFetch
 from rics.translation.offline import Format, TranslationMap
 from rics.translation.offline.types import FormatType, PlaceholderTranslations, SourcePlaceholderTranslations
@@ -38,16 +40,18 @@ _NAME_ATTRIBUTES = ("name", "columns", "keys")
 LOGGER = logging.getLogger(__package__).getChild("Translator")
 
 FetcherTypes = Union[
-    Fetcher[SourceType, IdType],
     TranslationMap[NameType, SourceType, IdType],
+    Fetcher[SourceType, IdType],
     SourcePlaceholderTranslations[SourceType],
     Dict[SourceType, PlaceholderTranslations.MakeTypes],
 ]
 
 
-class Translator(Generic[Translatable, NameType, SourceType, IdType]):
+class Translator(Generic[NameType, SourceType, IdType]):
     """Translate IDs to human-readable labels.
 
+    For an introduction to translation, see the :ref:`translation-primer` page.
+
     The recommended way of initializing ``Translator`` instances is the :meth:`from_config` method. For configuration
     file details, please refer to the :ref:`translator-config` page.
 
@@ -84,7 +88,7 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
                1999 | Sofia   | Female        1 | Morris | true
                1904 | Fred    | Male          2 | Simba  | true
 
-        In most real cases we'd fetch this table from somewhere. In this case, howeever, there's so little data that we
+        In most real cases we'd fetch this table from somewhere. In this case, however, there's so little data that we
         can simply enumerate the components needed for translation ourselves to create a :class:`.MemoryFetcher`.
 
         >>> from rics.translation import Translator
@@ -128,25 +132,26 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
             '0:Bob, nice=Maybe?'
 
         Since we didn't give an explicit `default_fmt_placeholders`, the regular `fmt` is used instead. Formats can be
-        plain strings, in which case tranlation will never explicitly fail unless the name itself fails to map and
+        plain strings, in which case translation will never explicitly fail unless the name itself fails to map and
         :attr:`.Mapper.unmapped_values_action` is set to :attr:`.ActionLevel.RAISE`.
     """
 
     def __init__(
         self,
-        fetcher: FetcherTypes = None,
+        fetcher: FetcherTypes[NameType, SourceType, IdType] = None,
         fmt: FormatType = "{id}:{name}",
         mapper: Mapper[NameType, SourceType, None] = None,
         default_fmt: FormatType = None,
-        default_fmt_placeholders: MakeType = None,
+        default_fmt_placeholders: MakeType[SourceType, str, Any] = None,
         allow_name_inheritance: bool = True,
     ) -> None:
         self._fmt = fmt if isinstance(fmt, Format) else Format(fmt)
+        self._default_fmt_placeholders: Optional[InheritedKeysDict[SourceType, str, Any]]
         self._default_fmt_placeholders, self._default_fmt = _handle_default(
             self._fmt, default_fmt, default_fmt_placeholders
         )
 
-        self._cached_tmap: TranslationMap = TranslationMap({})
+        self._cached_tmap: TranslationMap[NameType, SourceType, IdType] = TranslationMap({})
         self._fetcher: Fetcher[SourceType, IdType]
         if fetcher is None:
             from rics.translation.testing import TestFetcher, TestMapper
@@ -175,35 +180,48 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
         else:
             raise TypeError(type(fetcher))  # pragma: no cover
 
-        self._mapper: Mapper = mapper or Mapper()
+        self._mapper: Mapper[NameType, SourceType, None] = mapper or Mapper()
 
         # Misc config
         self._allow_name_inheritance = allow_name_inheritance
 
+        self._config_metadata: Optional[_config_utils.ConfigMetadata] = None
+
     @classmethod
     def from_config(
         cls,
         path: PathLikeType,
-        extra_fetchers: Iterable[str] = (),
-    ) -> "Translator":
+        extra_fetchers: Iterable[PathLikeType] = (),
+        clazz: Union[str, Type["Translator[NameType, SourceType, IdType]"]] = None,
+    ) -> "Translator[NameType, SourceType, IdType]":
         """Create a ``Translator`` from TOML inputs.
 
         Args:
-            path: Path to a TOML file, or a pre-parsed dict.
+            path: Path to a TOML file.
             extra_fetchers: Path to TOML files defining additional fetchers. Useful for fetching from multiple sources
                 or kinds of sources, for example locally stored files in conjunction with one or more databases. The
-                fetchers are ranked by input order, with the fetcher defined in `path` being given the highest priority
-                (rank 0).
+                fetchers are ranked by input order, with the fetcher defined in `path` (if any) being given the highest
+                priority (rank 0).
+            clazz: Translator implementation to create. If a string is passed, the class is resolved using
+                :meth:`~rics.utility.misc.get_by_full_name` if a string is given. Use ``cls`` if ``None``.
 
         Returns:
-            A ``Translator`` instance.
-
-        See Also:
-            The :ref:`translator-config` page.
+            A new ``Translator`` instance with a :attr:`config_metadata` attribute.
         """
-        return factory.TranslatorFactory(path, extra_fetchers).create()
+        return factory.TranslatorFactory(
+            path,
+            extra_fetchers,
+            clazz or cls,  # TODO: Higher-Kinded TypeVars
+        ).create()
 
-    def copy(self, share_fetcher: bool = True, **overrides: Any) -> "Translator":
+    @property
+    def config_metadata(self) -> _config_utils.ConfigMetadata:
+        """Return :func:`from_config` initialization metadata."""
+        if self._config_metadata is None:
+            raise ValueError("Not created using Translator.from_config()")  # pragma: no cover
+        return self._config_metadata
+
+    def copy(self, share_fetcher: bool = True, **overrides: Any) -> "Translator[NameType, SourceType, IdType]":
         """Make a copy of this ``Translator``.
 
         Args:
@@ -235,21 +253,23 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
         if "fetcher" not in kwargs:
             kwargs["fetcher"] = self.fetcher if self.online else self._cached_tmap.copy()
 
-        return Translator(**kwargs)
+        return type(self)(**kwargs)
 
     def translate(
         self,
         translatable: Translatable,
-        names: NameTypes = None,
-        ignore_names: Names = None,
+        names: NameTypes[NameType] = None,
+        ignore_names: Names[NameType] = None,
         inplace: bool = False,
-        override_function: ExtendedOverrideFunction = None,
+        override_function: ExtendedOverrideFunction[NameType, SourceType, IdType] = None,
         maximal_untranslated_fraction: float = 1.0,
         reverse: bool = False,
         attribute: str = None,
     ) -> Optional[Translatable]:
         """Translate IDs to human-readable strings.
 
+        For an introduction to translation, see the :ref:`translation-primer` page.
+
         Args:
             translatable: A data structure to translate.
             names: Explicit names to translate. Derive from `translatable` if ``None``.
@@ -275,15 +295,13 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
         Raises:
             UntranslatableTypeError: If ``type(translatable)`` cannot be translated.
             AttributeError: If `names` are not given and cannot be derived from `translatable`.
-            MappingError: If required (explicitly given) names fail to map to a source.
+            MappingError: If any required (explicitly given) names fail to map to a source.
             MappingError: If name-to-source mapping is ambiguous.
             ValueError: If `maximal_untranslated_fraction` is not a valid fraction.
             TooManyFailedTranslationsError: If translation fails for more than `maximal_untranslated_fraction` of IDs.
             ConnectionStatusError: If ``reverse=True`` while the ``Translator`` is online.
-            UnknownSourceError: If `override_function` returns a source which is not known.
-
-        See Also:
-            The :meth:`.Mapper.apply` function, which performs both placeholder and name-to-source mapping.
+            UserMappingError: If `override_function` returns a source which is not known, and
+                ``self.mapper.unknown_user_override_action != 'ignore'``.
         """  # noqa: DAR101 darglint is bugged here
         if self.online and reverse:  # pragma: no cover
             raise ConnectionStatusError("Reverse translation cannot be performed online.")
@@ -328,25 +346,13 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
 
         return ans
 
-    def __call__(
-        self,
-        translatable: Translatable,
-        names: Names = None,
-        ignore_names: Names = None,
-        inplace: bool = False,
-    ) -> Optional[Translatable]:
-        """Inherits docstring from self.translate."""
-        return self.translate(translatable, names, ignore_names, inplace)  # pragma: no cover
-
-    __call__.__doc__ = translate.__doc__
-
     def map(  # noqa: A003
         self,
         translatable: Translatable,
-        names: NameTypes = None,
-        ignore_names: Names = None,
-        override_function: ExtendedOverrideFunction = None,
-    ) -> Optional[DirectionalMapping]:
+        names: NameTypes[NameType] = None,
+        ignore_names: Names[NameType] = None,
+        override_function: ExtendedOverrideFunction[NameType, SourceType, IdType] = None,
+    ) -> Optional[DirectionalMapping[NameType, SourceType]]:
         """Map names to translation sources.
 
         Args:
@@ -365,55 +371,72 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
 
         Raises:
             AttributeError: If `names` are not given and cannot be derived from `translatable`.
-            MappingError: If required (explicitly given) names fail to map to a source.
+            MappingError: If any required (explicitly given) names fail to map to a source.
             MappingError: If name-to-source mapping is ambiguous.
-            UnknownSourceError: If `override_function` returns a source which is not known.
+            UserMappingError: If `override_function` returns a source which is not known, and
+                ``self.mapper.unknown_user_override_action != 'ignore'``.
         """
         return self._map_inner(translatable, names, ignore_names=ignore_names, override_function=override_function)
 
+    def map_scores(
+        self,
+        translatable: Translatable,
+        names: NameTypes[NameType] = None,
+        ignore_names: Names[NameType] = None,
+        override_function: ExtendedOverrideFunction[NameType, SourceType, IdType] = None,
+    ) -> pd.DataFrame:
+        """Returns raw match scores for name-to-source mapping. See :meth:`map` for details."""
+        names_to_translate = self._resolve_names(translatable, names, ignore_names)
+
+        return self.mapper.compute_scores(
+            names_to_translate,
+            self.sources,
+            override_function=None if override_function is None else self._wrap_override_function(override_function),
+        )
+
+    @property
+    def sources(self) -> List[SourceType]:
+        """Return translation sources."""
+        return self.fetcher.sources if self.online else self._cached_tmap.sources
+
     def _map_inner(
         self,
         translatable: Translatable,
-        names: NameTypes = None,
-        ignore_names: Names = None,
-        override_function: ExtendedOverrideFunction = None,
+        names: NameTypes[NameType] = None,
+        ignore_names: Names[NameType] = None,
+        override_function: ExtendedOverrideFunction[NameType, SourceType, IdType] = None,
         parent: Translatable = None,
-    ) -> Optional[DirectionalMapping]:
+    ) -> Optional[DirectionalMapping[NameType, SourceType]]:
         names_to_translate = self._resolve_names(translatable, names, ignore_names, parent)
-        sources = self.fetcher.sources if self.online else self._cached_tmap.sources
 
-        def func(v: NameType, c: Set[SourceType], _: None) -> Optional[SourceType]:
-            assert override_function is not None, "This shouldn't happen"  # noqa: S101
-            res = override_function(v, c, [])
-            if res is None:
+        name_to_source = self.mapper.apply(
+            names_to_translate,
+            self.sources,
+            override_function=None if override_function is None else self._wrap_override_function(override_function),
+        )
+
+        unmapped = set() if names is None else set(as_list(names)).difference(name_to_source.left)
+        if unmapped or not name_to_source.left:
+            params_info = (
+                f"could not be mapped to sources={self.sources}. "
+                "Additional parameters: ("
+                f"{ignore_names=}, "
+                f"override_function={tname(override_function)}, "
+                f"parent={tname(parent)}"
+                ")"
+            )
+
+            if names is None:
+                derived_names = self._extract_from_attribute_or_parent(translatable, parent)
+                msg = f"Translation aborted; none of the derived names {derived_names} {params_info}."
+                warnings.warn(msg, MappingWarning)
+                LOGGER.warning(msg)
                 return None
-            if isinstance(res, dict):
-                raise NotImplementedError(
-                    "Name splitting is not yet supported. See https://github.com/rsundqvist/rics/issues/64"
-                )
-            else:
-                return res
-
-        if override_function is None:
-            name_to_source = self.mapper.apply(names_to_translate, sources)
-        else:
-            try:
-                name_to_source = self.mapper.apply(names_to_translate, sources, override_function=func)
-            except UserMappingError as e:
-                raise UnknownSourceError(e.value, e.candidates) from e
-
-        # Fail if any of the explicitly given (i.e. literal, not predicate) names fail to map to a source.
-        if isinstance(names, (int, str, Iterable)):
-            required = set(as_list(names))
-            unmapped = required.difference(name_to_source.left)
-            if unmapped:
-                raise MappingError(f"Required names {unmapped} not mapped with {sources=} and {ignore_names=}.")
-
-        if not name_to_source.left:
-            msg = f"Translation aborted since none of {names=} could be mapped with {sources=}"
-            warnings.warn(msg, MappingWarning)
-            LOGGER.warning(msg)
-            return None
+            elif unmapped:
+                # Fail if any of the explicitly given names fail to map to a source.
+                msg = f"Required names {unmapped} {params_info}."
+                LOGGER.error(msg)
+                raise MappingError(msg)
 
         if name_to_source.cardinality.many_right:  # pragma: no cover
             for value, candidates in name_to_source.left_to_right.items():
@@ -425,12 +448,33 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
 
         return name_to_source
 
+    @classmethod
+    def _wrap_override_function(
+        cls,
+        override_function: ExtendedOverrideFunction[NameType, SourceType, IdType],
+    ) -> UserOverrideFunction[NameType, SourceType, None]:
+        def func(v: NameType, c: Set[SourceType], _: None) -> Optional[SourceType]:
+            assert override_function is not None, "This shouldn't happen"  # noqa: S101
+            res = override_function(v, c, [])
+            if res is None:
+                return None
+            if isinstance(res, dict):
+                # This will probably be some kind of class in the future, as we need to do bookkeeping to keep track
+                # of where we should fetch our translations later.
+                raise NotImplementedError(
+                    "Name splitting is not yet supported. See https://github.com/rsundqvist/rics/issues/64"
+                )
+            else:
+                return res
+
+        return func
+
     def fetch(
         self,
         translatable: Translatable,
         name_to_source: DirectionalMapping[NameType, SourceType],
         data_structure_io: Type[DataStructureIO] = None,
-    ) -> TranslationMap:
+    ) -> TranslationMap[NameType, SourceType, IdType]:
         """Fetch translations.
 
         Args:
@@ -475,7 +519,73 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
         return self._cached_tmap
 
     @classmethod
-    def restore(cls, path: PathLikeType) -> "Translator":
+    def load_persistent_instance(
+        cls,
+        config_path: PathLikeType,
+        extra_fetchers: Iterable[PathLikeType] = (),
+        cache_dir: PathLikeType = None,
+        max_age: Union[str, pd.Timedelta, timedelta] = "7d",
+        clazz: Union[str, Type["Translator[NameType, SourceType, IdType]"]] = None,
+    ) -> "Translator[NameType, SourceType, IdType]":
+        """Load or create a persistent :attr:`~.Fetcher.fetch_all`-instance.
+
+        .. warning:: Experimental method; may change or disappear without warning.
+
+        Instances are created, stored and loaded as determined by a metadata file located in the given `cache_dir`. A
+        new ``Translator`` will be created if:
+
+        * There is no `'metadata'` file, or
+        * the original ``Translator`` is too old (see `max_age`), or
+        * the current configuration -- as defined by ``(config_path, extra_fetchers, clazz)`` -- has changed in such a
+          way that it is no longer :meth:`equivalent <rics.translation.ConfigMetadata.is_equivalent>` to the
+          configuration used to create the original ``Translator``.
+
+        .. note:: This method is **not** thread safe.
+
+        Args:
+            config_path: Path to a TOML file. See :meth:`from_config` for details.
+            extra_fetchers: Path to TOML files defining additional fetchers. See :meth:`from_config` for details.
+            cache_dir: Root directory where the cached translator and associated metadata is stored. Derive based on
+                `config_path` if ``None``.
+            max_age: The maximum age of the cached ``Translator`` before it must be recreated. Pass ``max_age=0`` to
+                force recreation.
+            clazz: Translator implementation to create. If a string is passed, the class is resolved using
+                :meth:`~rics.utility.misc.get_by_full_name`. Use ``cls`` if ``None``.
+
+        Returns:
+            A new or cached ``Translator`` instance with a :attr:`config_metadata` attribute.
+        """
+        warnings.warn("Method 'Translator.load_persistent_instance' is EXPERIMENTAL. Use with caution.", UserWarning)
+
+        path = Path(str(config_path))
+        cache_dir = (
+            Path.home().joinpath(".rics").joinpath("cache").joinpath(path)
+            if cache_dir is None
+            else Path(str(cache_dir))
+        )
+        cache_dir.mkdir(parents=True, exist_ok=True)
+
+        metadata_path = cache_dir.joinpath("metadata.json")
+        cache_path = cache_dir.joinpath("translator.pkl")
+
+        extra_fetcher_paths: List[str] = list(map(str, extra_fetchers))
+
+        reference_metadata = _config_utils.make_metadata(
+            str(path),
+            extra_fetcher_paths,
+            clazz=factory.TranslatorFactory.resolve_class(clazz or cls),
+        )
+        if _config_utils.use_cached_translator(metadata_path, reference_metadata, pd.Timedelta(max_age)):
+            return cls.restore(cache_path)
+        else:
+            ans: Translator[NameType, SourceType, IdType] = cls.from_config(path, extra_fetcher_paths, clazz).store(
+                path=cache_path, delete_fetcher=True
+            )
+            metadata_path.write_text(ans.config_metadata.to_json())
+            return ans
+
+    @classmethod
+    def restore(cls, path: PathLikeType) -> "Translator[NameType, SourceType, IdType]":
         """Restore a serialized ``Translator``.
 
         Args:
@@ -485,30 +595,34 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
             A ``Translator``.
 
         Raises:
-            TypeError: If the object at `path` is not a ``Translator``.
+            TypeError: If the object at `path` is not a ``Translator`` or a subtype thereof.
 
         See Also:
             The :meth:`Translator.store` method.
         """
         import pickle  # noqa: S403
 
-        path = str(Path(str(path)).expanduser())
-        with open(path, "rb") as f:
+        full_path = Path(str(path)).expanduser()
+        with full_path.open("rb") as f:
             ans = pickle.load(f)  # noqa: S301
 
-        if not isinstance(ans, Translator):  # pragma: no cover
-            raise TypeError(f"Serialized object at at {path=} is a {type(ans)}, not {Translator.__name__}.")
+        if not isinstance(ans, cls):  # pragma: no cover
+            raise TypeError(f"Serialized object at at '{full_path}' is a {type(ans)}, not {cls}.")
+
+        if LOGGER.isEnabledFor(logging.DEBUG):
+            extra = "" if ans._config_metadata is None else f" with {ans.config_metadata}"
+            LOGGER.debug(f"Deserialized {ans}{extra}.")
 
         return ans
 
     def store(
         self,
         translatable: Translatable = None,
-        names: Names = None,
-        ignore_names: Names = None,
+        names: NameTypes[NameType] = None,
+        ignore_names: Names[NameType] = None,
         delete_fetcher: bool = True,
         path: PathLikeType = None,
-    ) -> "Translator":
+    ) -> "Translator[NameType, SourceType, IdType]":
         """Retrieve and store translations in memory.
 
         Args:
@@ -534,7 +648,7 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
             The :meth:`Translator.restore` method.
         """
         if translatable is None:
-            source_translations: SourcePlaceholderTranslations = self._fetch(None)
+            source_translations: SourcePlaceholderTranslations[SourceType] = self._fetch(None)
             translation_map = self._to_translation_map(source_translations)
         else:
             maybe_none, _ = self._get_updated_tmap(translatable, names, ignore_names=ignore_names, force_fetch=True)
@@ -567,12 +681,12 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
     def _get_updated_tmap(
         self,
         translatable: Translatable,
-        names: NameTypes = None,
-        ignore_names: Names = None,
-        override_function: ExtendedOverrideFunction = None,
+        names: NameTypes[NameType] = None,
+        ignore_names: Names[NameType] = None,
+        override_function: ExtendedOverrideFunction[NameType, SourceType, IdType] = None,
         force_fetch: bool = False,
         parent: Translatable = None,
-    ) -> Tuple[Optional[TranslationMap], List[NameType]]:
+    ) -> Tuple[Optional[TranslationMap[NameType, SourceType, IdType]], List[NameType]]:
         """Get an updated translation map.  # noqa
 
         Setting ``force_fetch=True`` will ignore the cached translation if there is one.
@@ -601,15 +715,15 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
 
     def _get_ids_to_fetch(
         self,
-        name_to_source: DirectionalMapping,
+        name_to_source: DirectionalMapping[NameType, SourceType],
         translatable: Translatable,
         dio: Type[DataStructureIO],
-    ) -> List[IdsToFetch]:
+    ) -> List[IdsToFetch[SourceType, IdType]]:
         # Aggregate and remove duplicates.
         source_to_ids: Dict[SourceType, Set[IdType]] = {source: set() for source in name_to_source.right}
         n2s = name_to_source.flatten()  # Will fail if sources are ambiguous.
 
-        float_names: List[str] = []
+        float_names: List[NameType] = []
         num_coerced = 0
         for name, ids in dio.extract(translatable, list(n2s)).items():
             if len(ids) == 0:
@@ -618,7 +732,7 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
             if isinstance(ids[0], float):
                 float_names.append(name)
                 # Float IDs aren't officially supported, but is common when using Pandas since int types cannot be NaN.
-                # This is sometimes a problem for the build-in set (see https://github.com/numpy/numpy/issues/9358), and
+                # This is sometimes a problem for the built-in set (see https://github.com/numpy/numpy/issues/9358), and
                 # for several database drivers.
                 ids = numpy.unique(ids)  # type: ignore
                 keep_mask = ~numpy.isnan(ids)
@@ -630,14 +744,16 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
         if num_coerced > 100 and self.online:  # pragma: no cover
             warnings.warn(
                 f"To ensure proper fetcher operation, {num_coerced} float-type IDs have been coerced to integers. "
-                f"Enforcing supported data types (str and int) in your {tname(translatable)}-data may improve "
-                f"performance. Affected names ({len(float_names)}): {sorted(float_names)}."
+                f"Enforcing supported data types for IDs (str and int) in your {tname(translatable)}-data may improve "
+                f"performance. Affected names ({len(float_names)}): {float_names}."
                 "\nHint: Going offline will suppress this warning."
             )
 
         return [IdsToFetch(source, ids) for source, ids in source_to_ids.items()]
 
-    def _fetch(self, ids_to_fetch: Optional[List[IdsToFetch]]) -> SourcePlaceholderTranslations:
+    def _fetch(
+        self, ids_to_fetch: Optional[List[IdsToFetch[SourceType, IdType]]]
+    ) -> SourcePlaceholderTranslations[SourceType]:
         fetcher = self.fetcher
 
         placeholders = self._fmt.placeholders
@@ -654,7 +770,9 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
             else fetcher.fetch(ids_to_fetch, placeholders, required)
         )
 
-    def _to_translation_map(self, source_translations: SourcePlaceholderTranslations) -> TranslationMap:
+    def _to_translation_map(
+        self, source_translations: SourcePlaceholderTranslations[SourceType]
+    ) -> TranslationMap[NameType, SourceType, IdType]:
         return TranslationMap(
             source_translations,
             fmt=self._fmt,
@@ -666,7 +784,7 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
     def _verify_translations(
         translatable: Translatable,
         names_to_translate: List[NameType],
-        translation_map: TranslationMap,
+        translation_map: TranslationMap[NameType, SourceType, IdType],
         translatable_io: Type[DataStructureIO],
         maximal_untranslated_fraction: float,
     ) -> None:
@@ -702,26 +820,32 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
     def _resolve_names(
         self,
         translatable: Translatable,
-        names: NameTypes = None,
-        ignored_names: Names = None,
+        names: NameTypes[NameType] = None,
+        ignored_names: Names[NameType] = None,
         parent: Translatable = None,  # This isn't correct; should be different typevars.
     ) -> List[NameType]:
         if names is None:
-            if parent is None:
-                names = self._extract_from_attribute(translatable)
-            else:
-                try:
-                    names = self._extract_from_attribute(translatable)
-                except AttributeError:
-                    names = self._extract_from_attribute(parent)
-                    LOGGER.debug(
-                        f"Using {names=} from parent of type {tname(parent)} for child of type {tname(translatable)}"
-                    )
+            names = self._extract_from_attribute_or_parent(translatable, parent)
         else:
             names = as_list(names)
         ignored_names = ignored_names if callable(ignored_names) else set(as_list(ignored_names))
         return self._resolve_names_inner(names, ignored_names)
 
+    def _extract_from_attribute_or_parent(
+        self, translatable: Translatable, parent: Optional[Translatable]
+    ) -> List[NameType]:
+        if parent is None:
+            names = self._extract_from_attribute(translatable)
+        else:
+            try:
+                names = self._extract_from_attribute(translatable)
+            except AttributeError:
+                names = self._extract_from_attribute(parent)
+                LOGGER.debug(
+                    f"Using {names=} from parent of type {tname(parent)} for child of type {tname(translatable)}"
+                )
+        return names
+
     @classmethod
     def _extract_from_attribute(cls, translatable: Translatable) -> List[NameType]:
         no_use_keys = False
@@ -744,7 +868,7 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
 
     @classmethod
     def _resolve_names_inner(
-        cls, names: List[NameType], ignored_names: Union[NamesPredicate, Set[NameType]]
+        cls, names: List[NameType], ignored_names: Union[NamesPredicate[NameType], Set[NameType]]
     ) -> List[NameType]:
         predicate = _IgnoredNamesPredicate(ignored_names).apply
         names_to_translate = list(filter(predicate, names))
@@ -754,8 +878,12 @@ class Translator(Generic[Translatable, NameType, SourceType, IdType]):
 
 
 class _IgnoredNamesPredicate(Generic[NameType]):
-    def __init__(self, ignored_names: Union[NamesPredicate, Set[NameType]]) -> None:
-        self._func = ignored_names if callable(ignored_names) else ignored_names.__contains__
+    def __init__(self, ignored_names: Union[NamesPredicate[NameType], Set[NameType]]) -> None:
+        self._func: NamesPredicate[NameType]
+        if callable(ignored_names):
+            self._func = ignored_names
+        else:
+            self._func = ignored_names.__contains__
 
     def apply(self, name: NameType) -> bool:
         return not self._func(name)
@@ -764,12 +892,12 @@ class _IgnoredNamesPredicate(Generic[NameType]):
 def _handle_default(
     fmt: Format,
     default_fmt: Optional[FormatType],
-    default_fmt_placeholders: Optional[MakeType],
-) -> Tuple[Optional[InheritedKeysDict], Optional[Format]]:  # pragma: no cover
+    default_fmt_placeholders: Optional[MakeType[SourceType, str, Any]],
+) -> Tuple[Optional[InheritedKeysDict[SourceType, str, Any]], Optional[Format]]:  # pragma: no cover
     if default_fmt is None and default_fmt_placeholders is None:
         return None, None
 
-    dt: Optional[InheritedKeysDict] = None
+    dt: Optional[InheritedKeysDict[SourceType, str, Any]] = None
 
     if isinstance(default_fmt_placeholders, InheritedKeysDict):
         dt = default_fmt_placeholders
