@@ -42,10 +42,10 @@ class Fetcher(ABC, Generic[SourceType, IdType]):
     @abstractmethod
     def fetch(
         self,
-        ids_to_fetch: Iterable[IdsToFetch],
+        ids_to_fetch: Iterable[IdsToFetch[SourceType, IdType]],
         placeholders: Iterable[str] = (),
         required: Iterable[str] = (),
-    ) -> SourcePlaceholderTranslations:
+    ) -> SourcePlaceholderTranslations[SourceType]:
         """Retrieve placeholder translations from the source.
 
         Args:
@@ -72,7 +72,7 @@ class Fetcher(ABC, Generic[SourceType, IdType]):
         self,
         placeholders: Iterable[str] = (),
         required: Iterable[str] = (),
-    ) -> SourcePlaceholderTranslations:
+    ) -> SourcePlaceholderTranslations[SourceType]:
         """Fetch as much data as possible.
 
         Args:
