@@ -22,7 +22,7 @@ class SqlFetcher(AbstractFetcher[str, IdType]):
 
     Args:
         connection_string: A SQLAlchemy connection string. Read from environment variable if `connection_string` starts
-            with '@' followed by the name. Example: ``@TRANSLATION_DB_CONNECTION_STRING`` reads from
+            with `'@'` followed by the name. Example: ``@TRANSLATION_DB_CONNECTION_STRING`` reads from
             the `TRANSLATION_DB_CONNECTION_STRING` environment variable.
         password: Password to insert into the connection string. Will be escaped to allow for special characters. If
             given, the connection string must contain a password key, eg; ``dialect://user:{password}@host:port``. Can
@@ -31,6 +31,7 @@ class SqlFetcher(AbstractFetcher[str, IdType]):
         blacklist_tables: The only tables the ``SqlFetcher`` may not access. Mutually exclusive with `whitelist_tables`.
         include_views: If ``True``, discover views as well.
         fetch_all_limit: Maximum size of table to allow a fetch all-operation. 0=never allow. Ignore if ``None``.
+        engine_kwargs: A dict of keyword arguments for :func:`sqlalchemy.create_engine`.
         **kwargs: Primarily passed to ``super().__init__``, then used as :meth:`selection_filter_type` kwargs.
 
     Raises:
@@ -49,9 +50,10 @@ class SqlFetcher(AbstractFetcher[str, IdType]):
         connection_string: str,
         password: str = None,
         whitelist_tables: Iterable[str] = None,
-        blacklist_tables: Iterable[str] = None,
+        blacklist_tables: Iterable[str] = (),
         include_views: bool = True,
         fetch_all_limit: Optional[int] = 100_000,
+        engine_kwargs: Dict[str, Any] = None,
         **kwargs: Any,
     ) -> None:
         if kwargs:
@@ -65,17 +67,28 @@ class SqlFetcher(AbstractFetcher[str, IdType]):
             super_kwargs = {}
         super().__init__(**super_kwargs)
 
-        if whitelist_tables and blacklist_tables:
-            raise ValueError("At most one of whitelist and blacklist may be given.")  # pragma: no cover
-
-        self._engine = sqlalchemy.create_engine(self.parse_connection_string(connection_string, password))
-
-        self._whitelist = set(whitelist_tables or [])
-        self._blacklist = set(blacklist_tables or [])
-        self._table_ts_dict: Optional[Dict[str, SqlFetcher.TableSummary]] = None
+        self._engine = self.create_engine(connection_string, password, engine_kwargs or {})
+        self._estr = str(self.engine)
         self._reflect_views = include_views
         self._fetch_all_limit = fetch_all_limit
 
+        self._blacklist = set(blacklist_tables)
+
+        self._table_ts_dict: Optional[Dict[str, SqlFetcher.TableSummary]] = None
+
+        if whitelist_tables is None:
+            self._whitelist = set()
+        else:
+            if blacklist_tables:
+                raise ValueError("At most one of whitelist and blacklist may be given.")  # pragma: no cover
+
+            whitelist_tables = set(whitelist_tables)
+            if len(whitelist_tables) == 0:
+                self.close()
+                LOGGER.warning(f"Got empty 'whitelist_tables' argument. No tables are available to {self}.")
+
+            self._whitelist = set(whitelist_tables)
+
     @property
     def _summaries(self) -> Dict[str, "SqlFetcher.TableSummary"]:
         """Names and sizes of tables that the ``SqlFetcher`` may interact with."""
@@ -84,12 +97,14 @@ class SqlFetcher(AbstractFetcher[str, IdType]):
             ts_dict = self._get_summaries()
             if LOGGER.isEnabledFor(logging.INFO):
                 sz = {name: ts.size for name, ts in sorted(ts_dict.items())}
-                LOGGER.info(f"Processed {len(ts_dict)} tables in {format_perf_counter(start)}. Lengths={sz}.")
+                LOGGER.info(
+                    f"{self._estr}: Processed {len(ts_dict)} tables in {format_perf_counter(start)}. Lengths={sz}."
+                )
             self._table_ts_dict = ts_dict
 
         return self._table_ts_dict
 
-    def fetch_translations(self, instr: FetchInstruction) -> PlaceholderTranslations:
+    def fetch_translations(self, instr: FetchInstruction[str, IdType]) -> PlaceholderTranslations[str]:
         """Fetch columns from a SQL database."""
         ts = self._summaries[instr.source]
         columns = ts.select_columns(instr)
@@ -99,7 +114,7 @@ class SqlFetcher(AbstractFetcher[str, IdType]):
             raise exceptions.ForbiddenOperationError(self._FETCH_ALL, f"disabled for table '{ts.name}'.")
 
         stmt = select if instr.ids is None else self._make_query(ts, select, set(instr.ids))
-        return PlaceholderTranslations(instr.source, tuple(columns), tuple(self._engine.execute(stmt)))
+        return PlaceholderTranslations(instr.source, tuple(columns), tuple(self.engine.execute(stmt)))
 
     def _make_query(
         self, ts: "SqlFetcher.TableSummary", select: sqlalchemy.sql.Select, ids: Set[IdType]
@@ -131,19 +146,58 @@ class SqlFetcher(AbstractFetcher[str, IdType]):
     def allow_fetch_all(self) -> bool:
         return super().allow_fetch_all and all(s.fetch_all_permitted for s in self._summaries.values())
 
-    def __repr__(self) -> str:
-        engine = self._engine
-        tables = self.sources
-        return f"{tname(self)}({engine=}, {tables=})"
+    def __str__(self) -> str:
+        disconnected = "<disconnected>: " if not self.online else ""
+        return f"{tname(self)}({disconnected}{self._estr}, tables={repr(self.sources or '<no tables>')})"
+
+    @property
+    def engine(self) -> sqlalchemy.engine.Engine:
+        """Engine used by this fetcher.
+
+        Returns:
+            The ``Engine`` used for fetching.
+        """
+        self.assert_online()
+        return self._engine
 
     def close(self) -> None:  # pragma: no cover
-        LOGGER.info("Deleting %s", self._engine)
-        del self._engine
+        if self._engine is None:
+            return
+
+        LOGGER.debug("Dispose %s", self._estr)
+        self._table_ts_dict = {}
+        self._engine.dispose()
         self._engine = None
 
+    @classmethod
+    def create_engine(
+        cls,
+        connection_string: str,
+        password: Optional[str],
+        engine_kwargs: Dict[str, Any],
+    ) -> sqlalchemy.engine.Engine:
+        """Factory method used by ``__init__``.
+
+        For a more detailed description of the arguments and the behaviour of this function, see the
+        :class:`class docstring <SqlFetcher>`.
+
+        Args:
+            connection_string: A SQLAlchemy connection string. Read from environment variable if `connection_string`
+                starts with `'@'` followed by the name.
+            password: Password to insert into the connection string.
+            engine_kwargs: A dict of keyword arguments for :func:`sqlalchemy.create_engine`.
+
+        Returns:
+            A new ``Engine``.
+        """
+        return sqlalchemy.create_engine(
+            cls.parse_connection_string(connection_string, password),
+            **engine_kwargs,
+        )
+
     @classmethod
     def parse_connection_string(cls, connection_string: str, password: Optional[str]) -> str:  # pragma: no cover
-        """Parse a connection string. Read from environment if `connection_string` starts with '@'."""
+        """Parse a connection string. Read from environment if `connection_string` starts with `'@'`."""
         connection_string = read_env_or_literal(connection_string)
         if password:
             connection_string = connection_string.format(password=quote_plus(read_env_or_literal(password)))
@@ -153,7 +207,7 @@ class SqlFetcher(AbstractFetcher[str, IdType]):
         start = perf_counter()
         metadata = self.get_metadata()
         if LOGGER.isEnabledFor(logging.DEBUG):
-            LOGGER.debug(f"Metadata created in {format_perf_counter(start)}.")
+            LOGGER.debug(f"{self._estr}: Metadata created in {format_perf_counter(start)}.")
 
         table_names = set(metadata.tables)
         if self._whitelist:
@@ -163,13 +217,13 @@ class SqlFetcher(AbstractFetcher[str, IdType]):
 
         if not tables:  # pragma: no cover
             if self._whitelist:
-                extra = " (whitelist)"
+                extra = f" (whitelist: {self._whitelist})"
             elif self._blacklist:
                 extra = f" (blacklist: {self._blacklist})"
             else:
                 extra = ""
 
-            raise ValueError(f"No sources found{extra}. Available tables: {table_names}")
+            LOGGER.warning(f"{self._estr}: No sources found{extra}. Available tables: {table_names}")
 
         ans = {}
         for name in tables:
@@ -189,7 +243,7 @@ class SqlFetcher(AbstractFetcher[str, IdType]):
         start = perf_counter()
         size = self.get_approximate_table_size(table, id_column)
         if LOGGER.isEnabledFor(logging.DEBUG):
-            LOGGER.debug(f"Size of {table.name}={size} resolved in {format_perf_counter(start)}.")
+            LOGGER.debug(f"{self._estr}: Size of '{table.name}' resolved in {format_perf_counter(start)}: {size} rows.")
         fetch_all_permitted = self._fetch_all_limit is None or size < self._fetch_all_limit
         return SqlFetcher.TableSummary(table.name, size, table.columns, fetch_all_permitted, id_column)
 
@@ -210,11 +264,11 @@ class SqlFetcher(AbstractFetcher[str, IdType]):
         Returns:
             An approximate size for `table`.
         """
-        return self._engine.execute(sqlalchemy.func.count(id_column)).scalar()
+        return int(self.engine.execute(sqlalchemy.func.count(id_column)).scalar())
 
     def get_metadata(self) -> sqlalchemy.MetaData:
         """Create a populated metadata object."""
-        metadata = sqlalchemy.MetaData(self._engine)
+        metadata = sqlalchemy.MetaData(self.engine)
         metadata.reflect(only=self._whitelist or None, views=self._reflect_views)
         return metadata
 
@@ -293,6 +347,6 @@ class SqlFetcher(AbstractFetcher[str, IdType]):
         id_column: sqlalchemy.schema.Column
         """The ID column of the table."""
 
-        def select_columns(self, instr: FetchInstruction) -> List[str]:
+        def select_columns(self, instr: FetchInstruction[str, IdType]) -> List[str]:
             """Return required and optional columns of the table."""
             return support.select_placeholders(instr, self.columns.keys())
