@@ -1,16 +1,18 @@
 from copy import copy
-from typing import Any, Dict, Generic, Iterator, List, Mapping, Optional, Tuple, Type, Union
+from typing import Any, Dict, Generic, Iterator, List, Mapping, Optional, Set, Type, Union
 
 from rics.translation.offline import MagicDict
-from rics.translation.offline._format import Format, FormatType
+from rics.translation.offline._format import Format
 from rics.translation.offline._format_applier import DefaultFormatApplier, FormatApplier
-from rics.translation.offline.types import NameToSourceDict, SourcePlaceholderTranslations
+from rics.translation.offline.types import FormatType, SourcePlaceholderTranslations
 from rics.translation.types import IdType, NameType, SourceType
 from rics.utility.collections.dicts import InheritedKeysDict, reverse_dict
 from rics.utility.misc import tname
 
+NameToSource = Dict[NameType, SourceType]
 
-class TranslationMap(Mapping, Generic[NameType, SourceType, IdType]):
+
+class TranslationMap(Generic[NameType, SourceType, IdType], Mapping[Union[NameType, SourceType], MagicDict[IdType]]):
     """Storage class for fetched translations.
 
     Args:
@@ -25,13 +27,13 @@ class TranslationMap(Mapping, Generic[NameType, SourceType, IdType]):
         `mypy#3004 <https://github.com/python/mypy/issues/3004>`_
     """
 
-    FORMAT_APPLIER_TYPE: Type[FormatApplier] = DefaultFormatApplier
+    FORMAT_APPLIER_TYPE: Type[FormatApplier[NameType, SourceType, IdType]] = DefaultFormatApplier
     """Format application implementation type. Overwrite attribute to customize."""
 
     def __init__(
         self,
-        source_translations: SourcePlaceholderTranslations,
-        name_to_source: NameToSourceDict = None,
+        source_translations: SourcePlaceholderTranslations[SourceType],
+        name_to_source: NameToSource[NameType, SourceType] = None,
         fmt: FormatType = None,
         default_fmt: FormatType = None,
         default_fmt_placeholders: InheritedKeysDict[SourceType, str, Any] = None,
@@ -39,19 +41,21 @@ class TranslationMap(Mapping, Generic[NameType, SourceType, IdType]):
         self.default_fmt = default_fmt  # type: ignore
         self.default_fmt_placeholders = default_fmt_placeholders  # type: ignore
         self.fmt = fmt  # type: ignore
-        self._source_formatters: Dict[SourceType, FormatApplier] = {
-            source: TranslationMap.FORMAT_APPLIER_TYPE(translations)
-            for source, translations in source_translations.items()
+        self._source_formatters: Dict[SourceType, FormatApplier[NameType, SourceType, IdType]] = {
+            source: self.FORMAT_APPLIER_TYPE(translations) for source, translations in source_translations.items()
         }
+        self._names_and_sources: Set[Union[NameType, SourceType]] = set()
         self.name_to_source = name_to_source or {}
 
         self._reverse_mode: bool = False
 
-    def apply(self, name: NameType, fmt: FormatType = None, default_fmt: FormatType = None) -> MagicDict[IdType]:
+    def apply(
+        self, name_or_source: Union[NameType, SourceType], fmt: FormatType = None, default_fmt: FormatType = None
+    ) -> MagicDict[IdType]:
         """Create translations for names. Note: ``__getitem__`` delegates to this method.
 
         Args:
-            name: A name to translate.
+            name_or_source: A name or source to translate.
             fmt: Format to use. If ``None``, fall back to init format.
             default_fmt: Alternative format for default translation. Resolution: Arg -> init arg, fmt arg, init fmt arg
 
@@ -62,20 +66,21 @@ class TranslationMap(Mapping, Generic[NameType, SourceType, IdType]):
             ValueError: If ``fmt=None`` and initialized without `fmt`.
             KeyError: If trying to translate `name` which is not known.
         """
-        if fmt is None:  # pragma: no cover
-            if self._fmt is None:
-                raise ValueError("No format specified and none given at initialization.")
-            else:
-                fmt = self._fmt
+        fmt = self._fmt if fmt is None else fmt
+        if fmt is None:
+            raise ValueError("No format specified and None given at initialization.")  # pragma: no cover
 
         fmt = Format.parse(fmt)
         default_fmt = self._default_fmt if default_fmt is None else Format.parse(default_fmt)
-        source = self.name_to_source[name]
-        translations = self._source_formatters[source](
+        source = self.name_to_source.get(name_or_source, name_or_source)  # type: ignore
+        translations: MagicDict[IdType] = self._source_formatters[source](
             fmt, default_fmt=default_fmt, default_fmt_placeholders=self.default_fmt_placeholders.get(source)
         )
         return (
-            MagicDict(reverse_dict(translations), default_value=translations.default_value)
+            MagicDict(
+                reverse_dict(translations),  # type: ignore
+                default_value=None,  # force failure for unknown keys
+            )
             if self.reverse_mode
             else translations
         )
@@ -91,20 +96,19 @@ class TranslationMap(Mapping, Generic[NameType, SourceType, IdType]):
         return list(self._source_formatters)
 
     @property
-    def name_to_source(self) -> NameToSourceDict[NameType, SourceType]:
+    def name_to_source(self) -> NameToSource[NameType, SourceType]:
         """Return name-to-source mapping."""
         return self._name_to_source
 
     @name_to_source.setter
-    def name_to_source(self, value: NameToSourceDict) -> None:
-        """Update bindings. Mappings name->source are always added, but may be overridden by the user."""
-        source_to_source = {source: source for source in self.sources}
-        self._name_to_source: NameToSourceDict = {**source_to_source, **value}
+    def name_to_source(self, value: NameToSource[NameType, SourceType]) -> None:
+        self._name_to_source: NameToSource[NameType, SourceType] = value
+        self._names_and_sources = set(value).union(self._source_formatters)
 
     @property
     def fmt(self) -> Optional[Format]:
         """Return the translation format."""
-        return self._fmt  # pragma: no cover
+        return self._fmt
 
     @fmt.setter
     def fmt(self, value: Optional[FormatType]) -> None:
@@ -113,19 +117,19 @@ class TranslationMap(Mapping, Generic[NameType, SourceType, IdType]):
     @property
     def default_fmt(self) -> Optional[Format]:
         """Return the format specification to use instead of `fmt` for fallback translation."""
-        return self._default_fmt  # pragma: no cover
+        return self._default_fmt
 
     @default_fmt.setter
     def default_fmt(self, value: Optional[FormatType]) -> None:
         self._default_fmt = None if value is None else Format.parse(value)
 
     @property
-    def default_fmt_placeholders(self) -> InheritedKeysDict:
+    def default_fmt_placeholders(self) -> InheritedKeysDict[SourceType, str, Any]:
         """Return the default translations used for `default_fmt_placeholders` placeholders."""
         return self._default_fmt_placeholders
 
     @default_fmt_placeholders.setter
-    def default_fmt_placeholders(self, value: Optional[InheritedKeysDict]) -> None:
+    def default_fmt_placeholders(self, value: Optional[InheritedKeysDict[SourceType, str, Any]]) -> None:
         self._default_fmt_placeholders = InheritedKeysDict() if value is None else InheritedKeysDict.make(value)
 
     @property
@@ -147,15 +151,17 @@ class TranslationMap(Mapping, Generic[NameType, SourceType, IdType]):
         """Make a copy of this ``TranslationMap``."""
         return copy(self)
 
-    def __getitem__(self, item: Union[NameType, Tuple[NameType, FormatType]]) -> MagicDict:
-        name, fmt = item if isinstance(item, tuple) else (item, self._fmt)
-        return self.apply(name, fmt)
+    def __getitem__(self, name_or_source: Union[NameType, SourceType]) -> MagicDict[IdType]:
+        return self.apply(name_or_source)
 
     def __len__(self) -> int:
-        return len(self.names)
+        return len(self._names_and_sources)
 
-    def __iter__(self) -> Iterator[NameType]:
-        return iter(self.names)  # pragma: no cover
+    def __iter__(self) -> Iterator[Union[NameType, SourceType]]:
+        return iter(self._names_and_sources)
+
+    def __bool__(self) -> bool:
+        return bool(self._source_formatters)
 
     def __repr__(self) -> str:
         sources = ", ".join(
